import datetime

from django.db.models import Avg, Q, StdDev, Sum
from django.db.models.functions import Abs, TruncMonth


def history(transaction_set):
    if not transaction_set.exists():
        return None

    _transaction_month = transaction_set.values(month=TruncMonth("date")).order_by(
        "-date"
    )
    _first_month = _transaction_month.last()["month"]
    _last_month = _transaction_month.first()["month"]

    _history = _transaction_month.annotate(
        sum_p=Sum("value", filter=Q(value__gt=0)),
        sum_m=Sum("value", filter=Q(value__lt=0)),
        sum=Sum("value"),
    ).order_by("-month")

    _data = [
        _history.filter(month=datetime.date(y, m + 1, 1)).first()
        or {"month": datetime.date(y, m + 1, 1), "sum": 0}
        for y in range(
            _first_month.year,
            _last_month.year + 1,
        )
        for m in range(
            _first_month.month - 1 if _first_month.year == y else 0,
            _last_month.month if _last_month.year == y else 12,
        )
    ]

    return {
        "data": _data,
        "max": {
            "pm": max(
                _history.aggregate(
                    max=Avg("sum_p", default=0) + StdDev("sum_p", default=0),
                    min=Avg("sum_m", default=0) - StdDev("sum_m", default=0),
                ).values(),
            ),
            "sum": _history.aggregate(max=Avg(Abs("sum")) + StdDev(Abs("sum")))["max"],
        },
    }
